## Copyright (C) 2000  Kai Habel
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

## -*- texinfo -*-
## @deftypefn {Function File} {} voronoi (@var{X},@var{Y})
## @deftypefnx {Function File} {} voronoi (@var{X},@var{Y},"plotstyle")
## @deftypefnx {Function File} {[@var{vx}, @var{vy}] =} voronoi (@var{X},@var{Y})
## plots voronoi diagram of points @var{X},@var{Y}. 
## The voronoi facets with points at infinity are not drawn.
## [@var{vx}, @var{vy}] = voronoi(...) returns the vertices instead plotting the
## diagramm. plot (@var{vx}, @var{vy}) shows the voronoi diagram.
## 
## @end deftypefn
## @seealso{voronoin, delaunay, convhull} 

## Author: Kai Habel <kai.habel@gmx.de>
## First Release: 20/08/2000

function [...] = voronoi (x, y, plt)

	if (nargin < 2 || nargin > 3)
		usage ("voronoi (x, y)")
	endif
	
	if (nargin < 3)
		plt = "b;;";
		## if not specified plot blue solid lines
	endif

	lx = length (x);
	ly = length (y);

	if (lx != ly)
		error ("voronoi: arguments must be vectors of same length");
	endif

	[p, lst, infi] = __voronoi__ ([x(:),y(:)]);

	idx = find (!infi);
	ll = length (idx);
	k = 0;r = 1;

	for i = 1:ll
		k += length (nth (lst, idx(i)));
	endfor

	vx = zeros (2,k);
	vy = zeros (2,k);

	for i=1:ll
		fac = nth (lst, idx(i));
		lf = length(fac);
		fac = [fac, fac(1)];
		fst = fac(1:length(fac)-1);
		sec = fac(2:length(fac));
		vx(:,r:r+lf-1) = [p(fst,1),p(sec,1)]';
		vy(:,r:r+lf-1) = [p(fst,2),p(sec,2)]';
		r += lf;
	endfor

	## this should be replaced with
	## vx = unique(vx,"rows");
	## vy = unique(vy,"rows");
	## when available
	del_idx = __unique_rows__ (vx);
	vx(del_idx,:) = [];
	vy(del_idx,:) = [];
	##
	##

	if (nargout == 0)
		plot (vx, vy, plt);
	elseif (nargout == 2)
		vr_val(vx);
		vr_val(vy);
	else
		error ("only two or zero output arguments supported")
	endif

endfunction
